/*
  ______   ______   _    _  _        _  _     
 (_____ \ (_____ \ | |  / )| |      (_)| |    
  _____) ) _____) )| | / / | |       _ | | _  
 (_____ ( |  ____/ | |< <  | |      | || || \ 
       | || |      | | \ \ | |_____ | || |_) )
       |_||_|      |_|  \_)|_______)|_||____/ 
                                              

                      - Version 0.1 -
                                             RedKod Team
                                           www.redkod.com
Coder: R-e-D
Mail : r-e-d@redkod.com

Packets forging library designed for NT System.
Copyright (C) 2002 R-e-D

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

*/

#include "stdafx.h"
#include "packets.h"
#include "rpklib.h"

BOOL APIENTRY DllMain(HANDLE hModule, DWORD ul_reason_for_call, LPVOID lpReserved)
{
	switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
    }
    return TRUE;
}


/**********************************/
/* Initialize Winsock Version 2.2 */
/**********************************/

RPKLIB_API int InitWinsock(void)
{
 	  	WORD wVersionRequested; 
 	  	WSADATA WSAData; 						       
 	  	int err;
 	  	
 	  	wVersionRequested = MAKEWORD(2,2); 		       	
	  	err = WSAStartup(wVersionRequested, &WSAData); 
	  	
	  	if(err == 0)
			return(0);
		else
			return(err);
}

/***************************************/
/* Create a raw socket and return this */
/***************************************/

RPKLIB_API SOCKET InitSocket(void) 
{
   int sock;
   int optval = 1;
   
   sock = WSASocket(AF_INET, SOCK_RAW, IPPROTO_RAW, NULL, 0,0);
   if (sock == INVALID_SOCKET)
   {
      perror("WSASocket");
      exit(-1);
   }
    
   if (setsockopt(sock, IPPROTO_IP, 2, (char *)&optval, sizeof(optval)) == SOCKET_ERROR)
   {
	    perror("setsockopt");
	    exit(-1);
   }

   return(sock);
}

/**********************/
/* Send a ICMP Packet */
/**********************/

RPKLIB_API int SendIcmpPacket(int Socket, char *saddr, char *daddr, unsigned char type, unsigned char code)
{
	int ret;
	unsigned short packet_size, ip_version, ip_len;	
	struct iphdr *ip;
	struct icmphdr *icmp;
	struct hostent *she, *dhe;
	struct sockaddr_in shost, dhost;  	
	char *ptr = NULL, packet[32]; /* sizeof(struct iphdr) + sizeof(struct icmphdr) */
 
	/* Size of the packet */
	packet_size = sizeof(struct iphdr) + sizeof(struct icmphdr);  

    	dhost.sin_family  = AF_INET;
	
	she = gethostbyname(saddr);
	if(she == 0)
	{		
		perror("gethostbyname");
		return(-1);
	}
	memcpy(&shost.sin_addr.s_addr, she->h_addr, 4);
	
	dhe = gethostbyname(daddr);
	if(!dhe) 
	{
		perror("gethostbyname");
		return(-1);
	}
	memcpy(&dhost.sin_addr.s_addr, dhe->h_addr, 4);

	ip = (struct iphdr *)malloc(sizeof(struct iphdr));
	memset(ip, 0x0, sizeof(struct iphdr));
	ip_len = sizeof(struct iphdr) / sizeof(unsigned long);
    	ip_version = 4;

    	ip->verlen   = (ip_version << 4) | ip_len;
    	ip->tos      = 0;                         
    	ip->tot_len  = htons (sizeof (struct iphdr) + sizeof (struct icmphdr));
    	ip->id       = 1;                
    	ip->offset   = 0;             
    	ip->ttl      = 255;           
    	ip->protocol = IPPROTO_ICMP;
    	ip->saddr    = shost.sin_addr.s_addr;
    	ip->daddr    = dhost.sin_addr.s_addr;
	ip->checksum = 0;
	ip->checksum = in_cksum((unsigned short *)ip, sizeof(struct iphdr));

	icmp = (struct icmphdr *)malloc(sizeof(struct icmphdr));
	memset(icmp, 0x0, sizeof(struct icmphdr));

    	icmp->type     = type;
    	icmp->code     = code;
    	icmp->id       = (USHORT)GetCurrentProcessId(); 
    	icmp->seq      = (USHORT)GetCurrentProcessId(); 
    	icmp->checksum = 0;
    	icmp->checksum = in_cksum((unsigned short *)icmp, sizeof(struct icmphdr));
 
    	ZeroMemory(packet, sizeof(packet));
    	ptr = packet;
    	memcpy(ptr, ip, sizeof(struct iphdr));
    	ptr += sizeof(struct iphdr);
    	memcpy(ptr, icmp, sizeof(struct icmphdr));
    	ptr += sizeof(struct icmphdr); 

	ret = sendto(Socket, packet, sizeof(packet), 0x0, (struct sockaddr *)&dhost, sizeof(struct sockaddr_in));			
	if (ret == SOCKET_ERROR)   
	{
			perror("sendto");
	}

	free(ip);
	free(icmp);
    
    return(ret);
}

/*********************/
/* Send a UDP Packet */
/*********************/

RPKLIB_API int SendUdpPacket(int Socket, char *saddr, int sport, char *daddr, int dport, char *data)
{
	struct iphdr  *ip;
    	struct udphdr *udp;
	struct hostent *she, *dhe;
	struct sockaddr_in shost, dhost;  
	unsigned short ip_len, ip_version;
	char *packet, *ptr = NULL, *Data = NULL;
    	int ret, datalen = 0;

	dhost.sin_family = AF_INET;
	
	she = gethostbyname(saddr);
	if(she == 0)
	{		
		perror("gethostbyname");
		return(-1);
	}
	memcpy(&shost.sin_addr.s_addr, she->h_addr, 4);
	
	dhe = gethostbyname(daddr);
	if(!dhe) 
	{
		perror("gethostbyname");
		return(-1);
	}
	memcpy(&dhost.sin_addr.s_addr, dhe->h_addr, 4);

	if(data != 0)
    	{
        datalen = strlen(data);
		Data = (char *) malloc (datalen);
		strcpy (Data, data);
	}

    	/* Remplissage de l'entete IP */
    	ip = (struct iphdr *)malloc(sizeof(struct iphdr));
	memset(ip, 0x0, sizeof(struct iphdr));

	ip_len = sizeof(struct iphdr) / sizeof(unsigned long);
    	ip_version = 4;

	ip->verlen   = (ip_version << 4) | ip_len;
    	ip->tos      = 0;                         
    	ip->tot_len  = htons (sizeof (struct iphdr) + sizeof (struct udphdr) + datalen);
    	ip->id       = 1;                
    	ip->offset   = 0;             
    		ip->ttl      = 255;           
    	ip->protocol = IPPROTO_UDP;
    	ip->saddr    = shost.sin_addr.s_addr;
    	ip->daddr    = dhost.sin_addr.s_addr;
	ip->checksum = 0;
	ip->checksum = in_cksum((unsigned short *)ip, sizeof(struct iphdr));

    /* Remplissage de l'entete UDP */	
	udp = (struct udphdr *)malloc(sizeof(struct udphdr));
	memset(udp, 0x0, sizeof(struct udphdr));

    	udp->length  = htons(sizeof (struct udphdr) + datalen); 
	udp->srcport = htons(sport);
	udp->dstport = htons(dport);
	udp->checksum = 0;

	packet = (char *)malloc(sizeof(struct iphdr) + sizeof(struct udphdr) + datalen);

	ZeroMemory(packet, sizeof(packet));
    	ptr = packet;
    	memcpy(ptr, ip, sizeof(struct iphdr));
    	ptr += sizeof(struct iphdr);
    	memcpy(ptr, udp, sizeof(struct udphdr));
    	ptr += sizeof(struct udphdr);
	memcpy(ptr, Data, datalen);
	ptr += sizeof(Data);	

    	/* Envoi du paquet */
    	ret = sendto(Socket, packet, sizeof(struct iphdr) + sizeof(struct udphdr) + datalen, 0x0, (struct sockaddr *)&dhost, sizeof(struct sockaddr_in)); 
	if (ret == -1)
    	{
        perror ("sendto");
    	}
	
	free(ip);
	free(udp);
	free(packet);

	return(ret);    
} 

/*********************/
/* Send a tcp packet */
/*********************/

RPKLIB_API int SendTcpPacket(int Socket, unsigned long saddr, unsigned long daddr, unsigned short sport, 
							 unsigned short dport,	 unsigned long sequence, unsigned long ack_num,
							 unsigned char tcpflags, unsigned short windowsize)
{
	struct iphdr  *ip;
    	struct tcphdr *tcp;
    	struct pseudohdr *temp_header;
	struct sockaddr_in *sin;
    	unsigned short packet_size, ip_version, ip_len;    
    	char *packet, *ptr = NULL;
	int ret;

	packet_size = sizeof(struct iphdr) + sizeof(struct tcphdr);
    
	packet = (char *)malloc(packet_size);	
	memset(packet, 0x0, sizeof(packet));

	/* IP Header */
    	ip_version = 4;
    	ip_len = sizeof(struct iphdr) / sizeof(unsigned long);
	
	ip = (struct iphdr *)malloc(sizeof(struct iphdr));
	memset(ip, 0x0, sizeof(struct iphdr));

    	ip->verlen   = (ip_version << 4) | ip_len;
    	ip->tos      = 0;                         
    	ip->tot_len  = htons(packet_size); 
    	ip->id       = 1;                
    	ip->offset   = 0;             
    	ip->ttl      = 255;              
    	ip->protocol = IPPROTO_TCP;            
    	ip->saddr    = saddr;	
    	ip->daddr    = daddr;
	ip->checksum = 0 ;
	ip->checksum = in_cksum((unsigned short *)ip, sizeof(struct iphdr));

	/* TCP Header */
	tcp = (struct tcphdr *)malloc(sizeof(struct tcphdr));
	memset(tcp, 0x0, sizeof(struct tcphdr));

    	tcp->sport      = htons(sport);	
    	tcp->dport      = htons(dport) ;	 
    	tcp->seqnum     = htonl(sequence);	
    	tcp->acknum     = htonl(ack_num);
    	tcp->DataOffset = (5) << 4;
    	tcp->Flags      = tcpflags;
    	tcp->Window     = htons(windowsize);
    	tcp->Checksum   = 0;
    	tcp->UrgPointer = 0;
   
   	 /* Pseudo Header */
	temp_header = (struct pseudohdr *)malloc(sizeof(struct pseudohdr));
	memset(temp_header, 0x0, sizeof(struct pseudohdr));

	temp_header->saddr    = saddr;
    	temp_header->daddr    = daddr;
    	temp_header->useless  = 0;
    	temp_header->protocol = IPPROTO_TCP;
    	temp_header->length   = htons(sizeof(struct tcphdr));
	temp_header->tcp      = *tcp;
    
    	tcp->Checksum = in_cksum((unsigned short *)temp_header, sizeof(struct pseudohdr));
    
	/* sockaddr_in structure */
	sin = (struct sockaddr_in *)malloc(sizeof(struct sockaddr_in));
	memset(sin, 0x0, sizeof(struct sockaddr_in));

	sin->sin_addr.s_addr = ip->daddr;
	sin->sin_family = AF_INET;

	/* The packet */
    	ZeroMemory(packet, sizeof(packet));
    	ptr = packet;	
    	memcpy(ptr, ip, sizeof(struct iphdr));
    	ptr += sizeof(struct iphdr);
    	memcpy(ptr, tcp, sizeof(struct tcphdr));
    	ptr += sizeof(struct tcphdr);	
	
	ret = sendto(Socket, packet, packet_size, 0x0, (struct sockaddr *)sin, sizeof(struct sockaddr_in)); 
	if (ret == -1)
    	{
        perror ("sendto");
    	}
	
	/* free all */
	free(packet);
	free(ip);
	free(tcp);
	free(temp_header);
	free(sin);

	return(ret);
}



/**************************/
/* Calculate the checksum */
/**************************/

unsigned short in_cksum (u_short *addr, int len)
{
  register int nleft = len;
  register u_short *w = addr;
  register int sum = 0;
  u_short answer = 0;

  while (nleft > 1)  
  {    
      sum += *w++; 
      nleft -= 2;
  }
   
  if (nleft == 1)
  {
      *(u_char *) (&answer) = *(u_char *) w;
      sum += answer; 
  }
  
  sum = (sum >> 16) + (sum & 0xffff);   
  sum += (sum >> 16);           
  answer = ~sum;     
  
  return (answer);
}	